package database;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import toolbox.GossipLog;

public class GossipAnalysisXMLDB extends XMLDB {
	
	public GossipAnalysisXMLDB(String dir, String path, String root) {
		super(dir, path, root);
	}
	
	public synchronized void addGossipLog(GossipLog gl) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			Element element0 = new Element("statusId");
			element0.setAttribute("statusId", gl.getStatusId());

			Element element1 = new Element("sent");
			element1.setText(String.valueOf(gl.getSent()));
			
			Element element2 = new Element("recv");
			element2.setText(String.valueOf(gl.getRecv()));
			
			Element element3 = new Element("alive");
			element3.setText(String.valueOf(gl.getAlive()));
			
			Element element4 = new Element("time");
			element4.setText(String.valueOf(gl.getTime()));
			
			element0.addContent(element1);
			element0.addContent(element2);
			element0.addContent(element3);
			element0.addContent(element4);

			root.addContent(0, element0);
			//root.addContent(findIndex(root, Long.valueOf(tl.getTime())), element0);
			doc.setRootElement(root);

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));

		} catch(JDOMException e1) {
			System.out.println("JDOMException @ addGossipLog in GossipAnalysisXMLDB, let's start a new one");
			xmlFailureHandler.setupNewFile();
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ addGossipLog in GossipAnalysisXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ addGossipLog in GossipAnalysisXMLDB");
		}
	}
	
	@SuppressWarnings("unchecked")
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				System.out.println(e.getAttributeValue("statusId")+": "+e.getChildText("sent")+" "+e.getChildText("recv"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ xmlParsePrint in GossipAnalysisXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ xmlParsePrint in GossipAnalysisXMLDB");
		}
	}
	
	@SuppressWarnings("unchecked")
	public synchronized int getSent(String sId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				if(e.getAttributeValue("statusId").compareTo(sId) == 0) {
					Integer i = Integer.valueOf(e.getChildText("sent"));
					return i;
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ xmlParsePrint in GossipAnalysisXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ xmlParsePrint in GossipAnalysisXMLDB");
		}
		return -1;
	}
	
	@SuppressWarnings("unchecked")
	public synchronized void increaseRecvMsg(String stId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for(Element e : list) {
				if(e.getAttributeValue("statusId").compareTo(stId) == 0) {
					int n = Integer.valueOf( e.getChildText("recv") ).intValue();
					n += 1;
					//System.out.println("add!! n = " + n);
					//e.getChild("sent").setText("shit");
					e.getChild("recv").setText(String.valueOf(n));
				}
			}
			
			XMLOutputter out = new XMLOutputter();
			out.output(document, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ increaseRecvMsg in GossipAnalysisXMLDB");
			xmlFailureHandler.setupNewFile();
		} catch (IOException e2) {
			System.out.println("IOException @ increaseRecvMsg in GossipAnalysisXMLDB");
		}
	}
	
	@SuppressWarnings("unchecked")
	public synchronized void increaseSentMsg(String stId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for(Element e : list) {
				if(e.getAttributeValue("statusId").compareTo(stId) == 0) {
					int n = Integer.valueOf( e.getChildText("sent") ).intValue();
					n = n + 1;
					//System.out.println("add!! n = " + n);
					//e.getChild("sent").setText("shit");
					e.getChild("sent").setText(String.valueOf(n));
				}
			}
			
			XMLOutputter out = new XMLOutputter();
			out.output(document, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ increaseRecvMsg in GossipAnalysisXMLDB");
			xmlFailureHandler.setupNewFile();
		} catch (IOException e2) {
			System.out.println("IOException @ increaseRecvMsg in GossipAnalysisXMLDB");
		}
	}
	/*
	public synchronized LinkedList<GossipLog> getGossipLogs(long time, int num) {
		LinkedList<GossipLog> ll = new LinkedList<GossipLog>();
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for(Element e : list) {
				if(num > 0) {
					long t = Long.valueOf(e.getChildText("time")).longValue();
					if(t > time) {
						ll.add( new GossipLog(t,
									e.getAttributeValue("statusId"), 
									Integer.valueOf(e.getChildText("sent")).intValue(),
									Integer.valueOf(e.getChildText("recv")).intValue(),
									Integer.valueOf(e.getChildText("alive")).intValue()));
					}
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getLastGossipLog in GossipAnalysisXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getLastGossipLog in GossipAnalysisXMLDB");
		}
		return ll;
	}
	
	public synchronized GossipLog getLastGossipLog() {
		GossipLog r = null;
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			if(list == null || list.size() == 0) {
				return null;
			}
			Element e = list.get(0);
			r = new GossipLog(Long.valueOf(e.getChildText("time")).longValue(),
					e.getAttributeValue("statusId"), 
					Integer.valueOf(e.getChildText("sent")).intValue(),
					Integer.valueOf(e.getChildText("recv")).intValue(),
					Integer.valueOf(e.getChildText("alive")).intValue());
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getLastGossipLog in GossipAnalysisXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getLastGossipLog in GossipAnalysisXMLDB");
		}
		return r;
	}
	*/
	/*
	//just for testing
	public synchronized void periodAdding() {
		while(true) {
			double lastValue = 20;
			int id = 0;
    		double factor1 = 0.90 + 0.2 * Math.random();
            int x1 = (int) (lastValue * factor1);
            //sentPeer.add(new Year(i), this.lastValue);
            
            double factor2 = 0.90 + 0.2 * Math.random();
            int x2 = (int) (x1 * factor2);
            //recvPeer.add(new Year(i), this.lastValue);
            
            double factor3 = 0.90 + 0.2 * Math.random();
            int x3 = (int) (x2 * factor3);
            //livePeer.add(new Year(i), this.lastValue);
            
            //dummy1.add(new Year(i), 0.0);
            //dummy2.add(new Year(i++), 0.0);
            id = id + 1;
            addGossipLog(new GossipLog(new Date().getTime(), String.valueOf(id), x1, x2, x3));
            
            try {
            	Thread.sleep(3000);
            } catch(Exception e) {}
    	}
	}
	*/
	
	public static void main(String[] args) {
		String dir = "xml";
    	GossipAnalysisXMLDB gossipDB = new GossipAnalysisXMLDB(dir, dir+"/gossipanalysis.xml", "gossipanalysis");
    	//gossipDB.periodAdding();
    	gossipDB.increaseSentMsg("1");
    	gossipDB.increaseRecvMsg("1");
	}
}
