package database;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.XMLEvent;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import application.Cuckoo;

import toolbox.HealthIndex;
import toolbox.UTPair;

public class PartnerXMLDB extends XMLDB {
	
	public PartnerXMLDB(String dir, String path, String root) {
		super(dir, path, root);
	}
	
	/*
	public synchronized void initPartnerList(PartnerMap partnerMap) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			
			for (Element e : list) {
				String fid = e.getAttributeValue("followeeId");
				String ids = e.getChildText("partnerIdList");
				String healths = e.getChildText("healthList");
				String sims    = e.getChildText("simList");
				
				String[] isplit = ids.split("#");
				String[] hsplit = healths.split("#");
				String[] ssplit = sims.split("#");
				for(int i=0; i<isplit.length; i++) {
					partnerMap.initAdd(fid, new Partner(fid, isplit[i], "", Integer.valueOf(hsplit[i]).intValue(), Integer.valueOf(ssplit[i]).intValue()));
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ initPartnerList in PartnerXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ initPartnerList in PartnerXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ initPartnerList in PartnerXMLDB");
		}
	}*/
	
	@SuppressWarnings("unchecked")
	public synchronized void xmlParsePrint() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				System.out.println("----------- partner ------------- ");
				System.out.println(e.getAttributeValue("followeeId"));
				System.out.println(e.getChildText("partnerIdList"));
				System.out.println(e.getChildText("healthList"));
				System.out.println(e.getChildText("number"));
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
	}
	
	@SuppressWarnings("unchecked")
	public synchronized List<String> getPartners(String uid) {
		LinkedList<String> partnerList = new LinkedList<String>();
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			XMLEvent event = null;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("followeeId")==0) {
					Iterator<Attribute> iterator = event.asStartElement().getAttributes();
					String fid = iterator.next().getValue();
					
					if(fid.compareTo(uid) == 0) {
						reader.nextEvent(); //<number>
						reader.nextEvent(); //<>
						reader.nextEvent(); //</number>
						reader.nextEvent(); //<partnerIdList>
						event = reader.nextEvent();
						//get partnerList
						if(event.isCharacters()) {
							String plist = event.asCharacters().getData();
							String[] split = plist.split("#");
							for(int i=0; i<split.length; i++) {
								partnerList.add(split[i]);
							}
						}
						in.close();
						return partnerList;
					}
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		} 
		return partnerList;
	}
	
	@SuppressWarnings("unchecked")
	public synchronized void refreshIndex(HashMap<String, HashMap<String, UTPair>> indice) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);
			Element root = doc.getRootElement();
			
			List<Element> list = root.getChildren();
			for (Element e : list) {
				String fid = e.getAttributeValue("followeeId");
				HashMap<String, UTPair> utpmap = indice.get(fid);
				if(utpmap != null) {
					//String pidList = e.getChildText("partnerIdList");
					//String hlthList = e.getChildText("healthList");
					replace(e, utpmap);
				}
			}
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException in xmlParsePrint!");
		} catch(IOException e2) {
			System.out.println("IOException in xmlParsePrint!");
		}
	}
	
	private synchronized void replace(Element e, HashMap<String, UTPair> map) {
		//0. initialization
		int pnumber = Integer.valueOf(e.getChildText("number"));
		String pidList = e.getChildText("partnerIdList");
		String hlthList = e.getChildText("healthList");
		List<UTPair> logutplist = new LinkedList<UTPair>();
		List<UTPair> newutplist = new LinkedList<UTPair>();
		
		/*
		System.out.println("followeeId:  " + e.getAttributeValue("followeeId"));
		System.out.println("partnerlist: " + pidList);
		System.out.println("healthlist:  " + hlthList);
		System.out.println("pnumber:     " + pnumber);
		*/
		
		//1. get the logged utps and update according to the "map"
		if(pidList.compareTo("") != 0) {
			String[] psplit = pidList.split("#");
			String[] hsplit = hlthList.split("#");
			if(psplit.length != hsplit.length) {
				System.out.println("ERROR: not one-one corresponding!");
				System.exit(0);
			}
			for(int i=0; i<psplit.length; i++) {
				try {
					UTPair utp = new UTPair(psplit[i], 0, Integer.valueOf(hsplit[i]));
			
					if(map.containsKey(utp.userId)) {
						UTPair iod = map.get(utp.userId);
						utp.health += iod.health;
						map.remove(utp.userId);
					}
					logutplist.add(utp);
				}catch(Exception exception) {
					System.out.println("path = " + path);
					System.out.println("pidList = " + pidList);
					System.out.println("hlthList = " + hlthList);
					System.exit(0);
				}
			}
		}
		/*
		printUTPairList(logutplist);
		printHashMap(map);
		*/
		
		//2. get the utps in the memory
		Set<String> set = map.keySet();
		Iterator<String> iter = set.iterator();
		while(iter.hasNext()) {newutplist.add(map.get(iter.next()));}
		/*
		printUTPairList(logutplist);
		printUTPairList(newutplist);
		*/
		
		//3. sort and abandon the negative ones;
		Collections.sort(logutplist, new UTPairComparator());
		Collections.sort(newutplist, new UTPairComparator());
		
		//printUTPairList(logutplist);
		//printUTPairList(newutplist);
		
		logutplist = getPositive(logutplist);
		newutplist = getPositive(newutplist);
		
		//System.out.println("===============================");
		
		//printUTPairList(logutplist);
		//printUTPairList(newutplist);
		
		
		//4. choose strong old guys and new guys
		int numOfOldGuys = logutplist.size();
		int numOfNewGuys = newutplist.size();
		List<UTPair> oldguys = null, newguys = null;
		if(numOfOldGuys<pnumber) {
			logutplist.addAll(newutplist);
			if(numOfOldGuys+numOfNewGuys < pnumber) {
				oldguys = logutplist;
				newguys = new LinkedList<UTPair>();
			} else if(numOfOldGuys+numOfNewGuys < 2*pnumber) {
				oldguys = logutplist.subList(0, pnumber);
				newguys = logutplist.subList(pnumber, logutplist.size());
			} else if(numOfOldGuys+numOfNewGuys >= 2*pnumber) {
				oldguys = logutplist.subList(0, pnumber);
				newguys = logutplist.subList(pnumber, 2*pnumber);
			}
		} else if(numOfOldGuys>pnumber && numOfNewGuys<pnumber) {
			oldguys = logutplist.subList(0, pnumber);
			List<UTPair> tmp = logutplist.subList(pnumber, numOfOldGuys);
			newutplist.addAll(tmp);
			if(newutplist.size()>pnumber) {
				newguys = newutplist.subList(0, pnumber);
			} else {
				newguys = newutplist;
			}
		} else if(numOfOldGuys==pnumber && numOfNewGuys<pnumber) {
			oldguys = logutplist;
			newguys = newutplist;
		} else if(numOfOldGuys>=pnumber && numOfNewGuys>=pnumber) {
			oldguys = logutplist.subList(0, pnumber);
			newguys = newutplist.subList(0, pnumber);
		}
		
		//printUTPairList(oldguys);
		//printUTPairList(newguys);
		
		//5. write to the string
		pidList = "";
		hlthList = "";
		try {
		Iterator<UTPair> iterator = oldguys.iterator();
		while(iterator.hasNext()) {
			UTPair u = iterator.next();
			pidList += (u.userId+"#");
			hlthList += (u.health+"#");
		}
		
		iterator = newguys.iterator();
		while(iterator.hasNext()) {
			UTPair u = iterator.next();
			pidList += (u.userId+"#");
			hlthList += (u.health+"#");
		}
		
		//6. set the xml element
		e.getChild("partnerIdList").setText(pidList);
		e.getChild("healthList").setText(hlthList);
		} catch(java.lang.NullPointerException npe) {
			printPath();
			System.out.println("numOfOldGuys: " + numOfOldGuys);
			System.out.println("numOfNewGuys: " + numOfNewGuys);
			System.out.println("pnumber: " + pnumber);
			System.out.println("------------printUTPairList(oldguys)-------------------");
			printUTPairList(oldguys);
			System.out.println("------------printUTPairList(newguys)-------------------");
			printUTPairList(newguys);
			npe.printStackTrace();
			System.exit(0);
		}
	}
	
	public void printHashMap(HashMap<String, UTPair> map) {
		Set<String> fset = map.keySet();
		Iterator<String> iter = fset.iterator();
		while(iter.hasNext()) {
			String uid = iter.next();
			UTPair pair = map.get(uid);
			System.out.print("<" + pair.userId + ", " + pair.health + "> ");
		}
		System.out.println();
	}
	
	private List<UTPair> getPositive(List<UTPair> utplist) {
		//System.out.println("-----------------");
		//printUTPairList(utplist);
		int index = 0;
		Iterator<UTPair> iterator = utplist.iterator();
		while(iterator.hasNext()) {
			UTPair utp = iterator.next();
			if(utp.health < 0) {
				break;
			} else{
				index ++;
			}
		}
		return utplist.subList(0, index);
		//printUTPairList(utplist);
	}
	
	public synchronized void initWrite(String[] partners, String uid) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			
			String pstring = "", hstring = "";
			for(int i=0; i<partners.length; i++) {
				pstring += partners[i]+"#";
				hstring += HealthIndex.INIT_BLOOD + "#";
			}
			
			Element element = new Element("followeeId");
			element.setAttribute("followeeId", uid);
			Element element1 = new Element("number");
			element1.setText(String.valueOf(partners.length));
			Element element2 = new Element("partnerIdList");
			element2.setText(pstring);
			Element element3 = new Element("healthList");
			element3.setText(hstring);
		
			element.addContent(element1);
			element.addContent(element2);
			element.addContent(element3);
			root.addContent(element);
			
			doc.setRootElement(root);
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ writeBack in PartnerXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ writeBack in PartnerXMLDB");
		}
	}
	/*
	public synchronized void writeBack(HashMap<String, PartnerQueue> partnerMap) {
		try {
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();
			List<Element> list = root.getChildren();
			
			//First update the out-of-date elements in the XML-based DB
			String id;
			PartnerQueue pq;
			PartnerRecordBlock prBlock;
			for (Element e : list) {
				id = e.getAttributeValue("followeeId");
				pq = partnerMap.remove(id);
				if(pq != null && pq.getLength() != 0) {
					prBlock = new PartnerRecordBlock(pq);
					e.getChild("partnerIdList").setText(prBlock.getPartnerIdList());
					e.getChild("healthList").setText(prBlock.getHealthList());
					e.getChild("simList").setText(prBlock.getSimList());
				}
			}
			
			//others: but here is not used in the emulation
			if(partnerMap.size() != 0) {
				Set<String> keyset = partnerMap.keySet();
				Iterator<String> iterator = keyset.iterator();
			
				while(iterator.hasNext()) {
					id = iterator.next();
					pq = partnerMap.get(id);
					prBlock = new PartnerRecordBlock(pq);
				
					Element element = new Element("followeeId");
					element.setAttribute("followeeId", prBlock.getFolloweeId());
					Element element1 = new Element("partnerIdList");
					element1.setText(prBlock.getPartnerIdList());
					Element element2 = new Element("healthList");
					element2.setText(prBlock.getHealthList());
					Element element3 = new Element("simList");
					element3.setText(prBlock.getSimList());
				
					element.addContent(element1);
					element.addContent(element2);
					element.addContent(element3);
					root.addContent(element);
				}
			}
			
			doc.setRootElement(root);
			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
			
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ writeBack in PartnerXMLDB");
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException @ writeBack in PartnerXMLDB");
		} catch(IOException e3) {
			System.out.println("IOException @ writeBack in PartnerXMLDB");
		}
	}
	*/
	
	/*
	//Record Block, used to write back to the XML-based DB
	public class PartnerRecordBlock {
		private String followeeId;
		private String partnerIdList;  //user # as the delimiter
		private String healthList;     //user # as the delimiter
		private String simList;        //user # as the delimiter
		
		public PartnerRecordBlock(String fid, String list) {
			followeeId = fid;
			partnerIdList = list;
		}
		
		public PartnerRecordBlock(PartnerQueue pqueue) {
			followeeId = pqueue.getFolloweeId();
			partnerIdList = pqueue.partners2string();
			healthList = pqueue.healthes2string();
			simList = pqueue.sims2string();
			//pqueue.buildLists(partnerIdList, healthList, simList);
			//System.out.println("partnerIdList ---> " + partnerIdList);
		}
		
		public String getFolloweeId() {
			return followeeId;
		}
		
		public String getPartnerIdList() {
			return partnerIdList;
		}
		
		public String getHealthList() {
			return healthList;
		}
		
		public String getSimList() {
			return simList;
		}
	}
	*/
	
	public static void printUTPairList(List<UTPair> utplist) {
		Iterator<UTPair> iterator = utplist.iterator();
		while(iterator.hasNext()) {
			UTPair tmp = iterator.next();
			System.out.print( "<" + tmp.userId + ", " + tmp.health + ">");
		}
		System.out.println();
	}
	
	class UTPairComparator implements Comparator<UTPair> {
		public int compare(UTPair utp1, UTPair utp2) {
			if(utp1.health < utp2.health) {
				return 1;
			} else if(utp1.health > utp2.health) {
				return -1;
			} else {
				return 0;
			}
		}
	}
	
	//just for test
	public static void main(String[] args) {
		//PartnerXMLDB xmlDB = new PartnerXMLDB("xml", "xml/partner.xml", "Partner");
		//PartnerMap pl = new PartnerMap(xmlDB);
		/*LinkedList<String> ll = new LinkedList<String>();
		for(int i=0; i<100; i++) {
			ll.add(String.valueOf(i));
		}
		
		List<String> ll2 = ll.subList(0, 100);
		
		Cuckoo.printStringList(ll);
		Cuckoo.printStringList(ll2);*/
		
		
		/*pl.printPartnerMap(1);
		System.out.println("----------------------------");
		pl.printPartnerMap(2);
		System.out.println("----------------------------");
		
		pl.runtimeAdd("456", new Partner("456", "19", "", 5, 1));
		
		pl.printPartnerMap(1);
		System.out.println("----------------------------");
		pl.printPartnerMap(2);
		System.out.println("----------------------------");
		*/
		
		//pl.printPartnerMap(1);
		
		/*String s = "12#13#14#15#@";
		String[] split = s.split("@");
		System.out.println(split.length);
		System.out.println(split[0]);
		String[] s2 = split[0].split("#");
		System.out.println(s2.length);*/
		
		//xmlDB.xmlParsePrint();
		
		//////////////////////////a completely test for partner/////////////////////////////
		PartnerXMLDB xmlDB = new PartnerXMLDB("xml/103931059", "xml/103931059/partner.xml", "Partner");
		//xmlDB.xmlParsePrint();
		
		//Cuckoo.printStringList(xmlDB.getPartners("148658740"));
		//String[] ss = {"987", "9876", "98765", "987654", "9876543", "98765432", "987654321"};
		//xmlDB.initWrite(ss, "12345");
		//xmlDB.xmlParsePrint();
		//PartnerMap pm = new PartnerMap(xmlDB);
		/////////
		//pm.increaseHealth("10434312", "789");
		//pm.increaseHealth("10434312", "789");
		
		//pm.writeBack(PartnerMap.FOR_SEND);
		
		//xmlDB.xmlParsePrint();
		
		List<String> ll = xmlDB.getPartners("112764971");
		Cuckoo.printStringList(ll);
		
	}
}
