package database;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.LinkedList;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import javax.xml.stream.XMLEventReader;  
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.XMLEvent;  

import application.Status;

//**********************************************************************************//
//**  Use DOM to build and write the XML (like add)                               **//
//**  Use SAX to fast parse the contents (like check the existence of one entry)  **//
//**********************************************************************************//

public class StatusXMLDB extends XMLDB {
	public static final long TIME_DEV = 180000; //3 minutes
	public static final int DUP_CHECK_PERIOD = 100;
	
	public StatusXMLDB(String dir, String path, String root) {
		super(dir, path, root);
	}
	
	public synchronized void xmlParsePrint() {
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("statusId")==0) {
					System.out.print(get1status(event, reader).toString());
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
	}

	public synchronized void addStatus(Status status) {
		try {
			if(recentExist(status, DUP_CHECK_PERIOD) == true) {
				return;
			}
			
			SAXBuilder builder = new SAXBuilder();
			Document doc = builder.build(path);

			Element root = doc.getRootElement();

			Element element = new Element("statusId");
			element.setAttribute("statusId", status.getStatusId());

			Element element1 = new Element("username");
			element1.setText(status.getAuthor());
		
			Element element2 = new Element("userId");
			element2.setText(status.getAuthorId());

			Element element3 = new Element("time");
			element3.setText(Long.toString(status.getTime()));

			Element element4 = new Element("text");
			element4.setText(status.getText());

			element.addContent(element1);
			element.addContent(element2);
			element.addContent(element3);
			element.addContent(element4);

			root.addContent(findIndex(Long.valueOf(status.getTime())), element);
			doc.setRootElement(root);

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		} catch(JDOMException e1) {
			System.out.println("JDOMException in addStatus! Let's start a new one!");
			xmlFailureHandler.setupNewFile();
		} catch(FileNotFoundException e2) {
			System.out.println("FileNotFoundException in addStatus!");
		} catch(IOException e3) {
			System.out.println("IOException in addStatus!");
		}
	}
	
	private synchronized int findIndex(long t) {
		int r = 0;
		try {  
			InputStream in = new FileInputStream(path);  
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			long time = 0;
			
			while (reader.hasNext()) {
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("time")==0) {
					event = reader.nextEvent();
					
					if(event.isCharacters()) {
						time = Long.valueOf( event.asCharacters().getData() );
					} else {
						System.out.println("ERROR: not time attribute!!!");
						System.exit(0);
					}
					
					if(t >= time) {
						in.close();
						return r;
					}
					r ++;
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return r;

		/*
		List<Element> ll = r.getChildren();
		for (Element e : ll) {
			if(t >= Long.valueOf(e.getChildText("time"))) {
				return r.indexOf(e);
			}
		}
		return ll.size();*/
	}
	
	/*
	public synchronized void deleteStatus(long statusId) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document doc = builder.build(file);

			Element root = doc.getRootElement();

			List<Element> list = root.getChildren();

			for (Element e : list) {
				if (Integer.parseInt(e.getAttributeValue("statusId")) == statusId) {
					root.removeContent(e);
					break;// ?? What should we do?
				}
			}

			XMLOutputter out = new XMLOutputter();
			out.output(doc, new FileOutputStream(path));
		} catch (JDOMException e1) {
			System.out.println("JDOMException in deleteStatus!");
		} catch (IOException e2) {
			System.out.println("IOException in deleteStatus!");
		}
	}*/

	public synchronized Status[] getRecentStatuses(int bulk) {
		LinkedList<Status> sll = new LinkedList<Status>();
		Status[] statuses;
		int num = 0;
		try {  
			InputStream in = new FileInputStream(path);
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			while (reader.hasNext()) {
				if(num >= bulk) {
					break;
				}
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("statusId")==0) {
					sll.addLast(get1status(event, reader));
		
					num ++;
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		
		statuses = new Status[sll.size()];
		Iterator<Status> iterator = sll.iterator();
		int i = 0;
		while(iterator.hasNext()) {
			statuses[i] = iterator.next();
			i = i+1;
		}
		return statuses;
		
	}
	
	public synchronized Status[] getStatuses(long time) {
		int index = findIndex(time);
		return getRecentStatuses(index);
	}
	
	//the 4 things: statusId, text, username, userId, time, only username can be null, other can never be null!
	@SuppressWarnings("unchecked")
	private Status get1status(XMLEvent firstEvent, XMLEventReader reader) {
		
		XMLEvent event = firstEvent;
		String statusId = null; String userId = null; String text = null; String username = null; long time = 0;
		try {
			Iterator<Attribute> iterator = event.asStartElement().getAttributes();
			statusId = iterator.next().getValue();
		
			reader.nextEvent(); //<username>
			event = reader.nextEvent();
			if(event.isCharacters()) {
				username = event.asCharacters().getData();
				reader.nextEvent(); //</username>
			} else if(event.isEndElement() && event.asEndElement().getName().toString() == "username"){
				username = null;
			} else {
				System.out.println("ERROR: inconsistence in get1status()");
				System.exit(0);
			}
		
			reader.nextEvent(); //<userId>
		
			event = reader.nextEvent();
			if(event.isCharacters()) {
				userId = event.asCharacters().getData();
			} else {
				System.out.println("ERROR: not time attribute!!!");
				System.exit(0);
			}
		
			reader.nextEvent(); //</userId>
			reader.nextEvent(); //<time>
		
			event = reader.nextEvent();
			if(event.isCharacters()) {
				time = Long.valueOf(event.asCharacters().getData());
			} else {
				System.out.println("ERROR: not time attribute!!!");
				System.exit(0);
			}
		
			reader.nextEvent(); //</time>
			reader.nextEvent(); //<text>
		
			event = reader.nextEvent();
			if(event.isCharacters()) {
				text = event.asCharacters().getData();
			} else {
				System.out.println("ERROR: not time attribute!!!");
				System.exit(0);
			}
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return new Status(statusId, text, username, userId, time);
	}
	
	/*
	public synchronized Status[] getRecentStatus(int bulk) {
		LinkedList<Status> sll = new LinkedList<Status>();
		Status[] status;
		while(true) {
			try {
				SAXBuilder builder = new SAXBuilder();
				InputStream file = new FileInputStream(path);
				Document document = builder.build(file);
				Element root = document.getRootElement();
				List<Element> list = root.getChildren();
				int i = 0;
				for (Element e : list) {
					sll.addLast( new Status(
							e.getAttributeValue("statusId"),
							e.getChildText("text"),
							e.getChildText("username"),
							e.getChildText("userId"),
							Long.valueOf(e.getChildText("time")).longValue()
							));
					i += 1;
					if(i >= bulk) {
						break;
					}
				}
				break;
			} catch(JDOMException e1) {
				//System.out.println("JDOMException @ getRecentStatus in StatusXMLDB");
				continue;
			} catch (IOException e2) {
				System.out.println("IOException @ getRecentStatus in StatusXMLDB");
				continue;
			}
		}
		status = new Status[sll.size()];
		Iterator<Status> iterator = sll.iterator();
		int i = 0;
		while(iterator.hasNext()) {
			status[i] = iterator.next();
			i = i+1;
		}
		return status;
	}
	
	public synchronized Status[] getStatuses(long time) {
		LinkedList<Status> sll = new LinkedList<Status>();
		Status[] status;
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				long t = Long.valueOf(e.getChildText("time"));
				if(t > time) {
					sll.addLast(new Status(
							e.getAttributeValue("statusId"),
					        e.getChildText("text"),
							e.getChildText("username"),
							e.getChildText("userId"),
							Long.valueOf(e.getChildText("time")).longValue()
							));
				} else {
					break;
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getRecentStatus in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getRecentStatus in StatusXMLDB");
		}
		//Copy the list to the array
		if(sll.size() == 0) {
			return null;
		}
		status = new Status[sll.size()];
		Iterator<Status> iterator = sll.iterator();
		int i = 0;
		while(iterator.hasNext()) {
			status[i] = iterator.next();
			i = i+1;
		}
		return status;
	}
	*/
	
	public synchronized static void printStatuses(Status[] statuses) {
		for(int i=0; i<statuses.length; i++) {
			System.out.println(statuses[i].getStatusId()+" "+statuses[i].getAuthor()+" "+statuses[i].getText()+" "+statuses[i].getTime());
		}
	}
	
	public static void printStatuses(LinkedList<Status> statuses) {
		Iterator<Status> iterator = statuses.iterator();
		Status ts;
		while(iterator.hasNext()) {
			ts = iterator.next();
			System.out.println(ts.getStatusId()+" "+ts.getAuthor()+" "+ts.getText()+" "+ts.getTime());
		}		
	}
	
	public synchronized boolean recentExist(Status s) {
		return recentExist(s, DUP_CHECK_PERIOD);
	}
	
	public synchronized boolean recentExist(Status s, int period) {
		try {  
			InputStream in = new FileInputStream(path);  
			XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader(in);
			
			XMLEvent event = null;
			//String uid = null; long time = 0; String text = null;
			int r = 0;
			while (reader.hasNext()) {
				if(r > period) {
					in.close();
					return false;
				}
				
				event = reader.nextEvent();
				if (event.isStartElement() && event.asStartElement().getName().toString().compareTo("statusId")==0) { //in a status loop
					/*reader.nextEvent(); //<username>
					event = reader.nextEvent();
					if(event.isCharacters()) {
						reader.nextEvent(); //</username>
					} else if(event.isEndElement()) {}
					reader.nextEvent(); //<userId>
					event = reader.nextEvent();
					if(event.isCharacters()) {
						uid = event.asCharacters().getData();
					} else {
						System.out.println("ERROR in parsing xml @ recentExist 1");
						System.exit(0);
					}
					reader.nextEvent(); //</userId>
					reader.nextEvent(); //<time>
					event = reader.nextEvent();
					if(event.isCharacters()) {
						time = Long.valueOf( event.asCharacters().getData() );
					} else {
						System.out.println("ERROR in parsing xml @ recentExist 2");
						System.exit(0);
					}
					reader.nextEvent(); //</time>
					reader.nextEvent(); //<text>
					event = reader.nextEvent();
					if(event.isCharacters()) {
						text = event.asCharacters().getData();
					} else {
						System.out.println("ERROR in parsing xml @ recentExist 3");
						System.exit(0);
					}*/
					
					//if(s.getAuthorId().compareTo(uid)==0 && s.getText().compareTo(text)==0 /*&& (s.getTime()>time-TIME_DEV || s.getTime()<time+TIME_DEV)*/) {
					if(s.getStatusId().compareTo(getStatusId(event, reader)) == 0) {
						//System.out.println("**************************************");
						//System.out.println(text);
						//System.out.println(s.getText());
						//System.out.println(r);
						in.close();
						return true;
					} /*else {
						System.out.println("**************************************");
						System.out.println(text);
						System.out.println(s.getText());
					} */
					r ++;
				}
			}
			in.close();
		} catch (Exception e) {  
			e.printStackTrace();  
		}
		return false;
	}
	
	@SuppressWarnings("unchecked")
	private String getStatusId(XMLEvent event, XMLEventReader reader) {
		Iterator<Attribute> iterator = event.asStartElement().getAttributes();
		return iterator.next().getValue();
	}
	/*
	//Whether a status exists in the XML-based DB
	public synchronized boolean exist(Status s) {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			List<Element> list = root.getChildren();
			for (Element e : list) {
				//System.out.println("///" + s.getTime());
				//System.out.println("///" + e.getChildText("time"));
				//System.out.println("///" + s.getStatusId());
				//System.out.println("///" + e.getAttributeValue("statusId"));
				
				if(Long.valueOf(s.getTime()).longValue() > Long.valueOf(e.getChildText("time")).longValue()) {
					return false;
				} else if(Long.valueOf(s.getTime()).longValue() == Long.valueOf(e.getChildText("time")).longValue()) {
					if(s.getStatusId().compareTo(e.getAttributeValue("statusId"))==0) {
						return true;
					}
				}
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ exist in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ exist in StatusXMLDB");
		}
		
		return false;
	}
	*/
	public synchronized long getLastTime() {
		try {
			SAXBuilder builder = new SAXBuilder();
			InputStream file = new FileInputStream(path);
			Document document = builder.build(file);
			Element root = document.getRootElement();
			Element e = root.getChild("statusId");
			if(e != null) {
				return Long.valueOf(e.getChildText("time"));
			} else {
				return 0;
			}
		} catch(JDOMException e1) {
			System.out.println("JDOMException @ getLastTime in StatusXMLDB");
		} catch (IOException e2) {
			System.out.println("IOException @ getLastTime in StatusXMLDB");
		}
		//Never get here!
		return 0;
	}
	
	// Currently, we do not need update!!
	/**
	 * public static void updateStatus(int id) throws JDOMException, IOException
	 * { SAXBuilder builder = new SAXBuilder(); InputStream file = new
	 * FileInputStream("xml/po.xml"); Document doc = builder.build(file);
	 * 
	 * Element root = doc.getRootElement();
	 * 
	 * List<Element> list = root.getChildren(); for(Element e:list) {
	 * if(Integer.parseInt(e.getAttributeValue("ID")) == id) {
	 * System.out.println("---------We find it!-----------");
	 * e.getChild("username").setText("111111111");
	 * e.getChild("password").setText("password"); } }
	 * 
	 * XMLOutputter out = new XMLOutputter(); out.output(doc, new
	 * FileOutputStream("xml/po.xml")); }
	 */

	public static void main(String ars[]) throws JDOMException, IOException {
		/*String dir = "xml/" + "hello";
		
		StatusXMLDB xmlDB = new StatusXMLDB(dir, dir+"/status.xml", "myStatus");
		xmlDB.creatXmlDoc();
		xmlDB.addStatus(new Status("123", "fuck you", "Tianyin", "1243", 121212));
		xmlDB.addStatus(new Status("234", "fuck you", "Tianyin", "1243", 121223));
		xmlDB.addStatus(new Status("345", "fuck you", "Tianyin", "1243", 121234));
		xmlDB.addStatus(new Status("456", "fuck you", "Tianyin", "1243", 121245));
		xmlDB.xmlParsePrint();
		
		System.out.println("-------------------------------------");
		
		xmlDB.addStatus(new Status("567", "fuck you", "Tianyin", "1243", 121233));
		
		xmlDB.addStatus(new Status("234", "fuck you", "Tianyin", "1243", 121223));
		xmlDB.addStatus(new Status("345", "fuck you", "Tianyin", "1243", 121234));
		xmlDB.addStatus(new Status("456", "fuck you", "Tianyin", "1243", 121245));
		
		//printStatuses(xmlDB.getRecentStatus(3));
		xmlDB.xmlParsePrint();
		
		//xmlDB.addStatus(new Status("234", "fuck you", "Tianyin", "1243", "121223"));
		
		//System.out.println();
		//printStatuses(xmlDB.getStatuses(121000));
		
		//xmlDB.xmlParsePrint();
		 */
		
		String dir = "xml";
		StatusXMLDB xmlDB = new StatusXMLDB(dir, dir+"/followeestatus.xml", "followeeStatus");
		xmlDB.printPath();
		//long lasting = System.currentTimeMillis();  
		//xmlDB.xmlParsePrint();
		
		//StatusXMLDB.printStatuses(xmlDB.getStatuses(1281548094338L));
		//System.out.println("runtime:" + (System.currentTimeMillis() - lasting) + " millis");  
		//System.out.println(xmlDB.getLastTime());
		System.out.println(xmlDB.recentExist(new Status("0", "Gay continuar?como entrenador del Real Zaragoza la pr�xima temporada http://tinyurl.com/39gb6du #sports", "shit", "148658740", 1285623438057L), 20));
		//System.out.println(xmlDB.findIndex(1285623649187L));
		//Status[] s = xmlDB.getRecentStatus(20);
		//StatusXMLDB.printStatuses(s);
	}

}
