package toolbox;

import java.util.Calendar;

import message.CuckooMsg;
import message.DirectUpdateStatusMsg;
import message.GossipUpdateStatusMsg;
import message.RplStatusErrMsg;
import message.RplStatusMsg;
//import java.util.Date;

import database.TrafficAnalysisXMLDB;

import rice.p2p.commonapi.Message;
import servermsg.ServerMessage;

/**
 * Actually, now it's Message Recording rather than Traffic Analysis
 */

public class TrafficAnalysisRecorder {
	private SizeCalculator sizeCalculator = new SizeCalculator();
	private TrafficAnalysisXMLDB trafficXMLDB;
	private String myUserId;
	
	public TrafficAnalysisRecorder(TrafficAnalysisXMLDB db, String myid) {
		trafficXMLDB = db;
		myUserId = myid;
	}
	
	//for CuckooMsg
	public void record(Message msg, long recvTime, String mk, String io, String fid, String tid, String redt) {
		TrafficLog trafficLog = null;
		if(msg instanceof CuckooMsg) {
			if(msg instanceof DirectUpdateStatusMsg) {
				trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(msg), ((DirectUpdateStatusMsg)msg).getText().length(), ((CuckooMsg) msg).getSTime(), recvTime, ((CuckooMsg) msg).getHops(), ((CuckooMsg) msg).getType(), mk, io, fid, tid, redt, 0, 0);
			} else if(msg instanceof GossipUpdateStatusMsg) {
				long gtime = ((GossipUpdateStatusMsg)msg).getGossipTime() + Calendar.getInstance().getTimeInMillis() - ((CuckooMsg) msg).getSTime();
				int ghops  = ((GossipUpdateStatusMsg)msg).getGossipHops() + 1;
				trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(msg), ((GossipUpdateStatusMsg) msg).getText().length(), ((CuckooMsg) msg).getSTime(), recvTime, ((CuckooMsg) msg).getHops(), ((CuckooMsg) msg).getType(), mk, io, fid, tid, redt, gtime, ghops);
			} else if(msg instanceof RplStatusMsg) {
				System.out.println("+++++++++++++++++++++++++++++ " + myUserId + " +++++++++++++++++++++++++++++++");
				System.out.println("RplStatusMsg fromUserId: " + ((RplStatusMsg) msg).getFromUserId());
				System.out.println("RplStatusMsg toUserId: " +   ((RplStatusMsg) msg).getToUserId());
				System.out.println("RplStatusMsg content length: " + ((RplStatusMsg) msg).getContentLength());
				System.out.println("RplStatusMsg acHops: " + ((RplStatusMsg) msg).getAcHops());
				System.out.println("RplStatusMsg reHops: " + ((RplStatusMsg) msg).getReHops());
				System.out.println("RplStatusMsg acTime: " + ((RplStatusMsg) msg).getAcTime());
				System.out.println("RplStatusMsg reTime: " + ((RplStatusMsg) msg).getReTime());
				System.out.println("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
				trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(msg), ((RplStatusMsg) msg).getContentLength(), ((CuckooMsg) msg).getSTime(), recvTime, ((CuckooMsg) msg).getHops(), ((CuckooMsg) msg).getType(), mk, io, fid, tid, redt, ((RplStatusMsg) msg).getAcTime(), ((RplStatusMsg) msg).getAcHops());
			} else if(msg instanceof RplStatusErrMsg) {
				System.out.println("------------------------------ " + myUserId + " ------------------------------");
				System.out.println("RplStatusErrMsg fromUserId: " + ((RplStatusErrMsg) msg).getFromUserId());
				System.out.println("RplStatusErrMsg toUserId: " +   ((RplStatusErrMsg) msg).getToUserId());
				System.out.println("RplStatusErrMsg acHops: " + ((RplStatusErrMsg) msg).getAcHops());
				System.out.println("RplStatusErrMsg hops: " + ((RplStatusErrMsg) msg).getReHops());
				System.out.println("RplStatusErrMsg acTime: " + ((RplStatusErrMsg) msg).getAcTime());
				System.out.println("RplStatusErrMsg time: " + ((RplStatusErrMsg) msg).getReTime());
				System.out.println("-------------------------------------------------------------");
				trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(msg), 0, ((CuckooMsg) msg).getSTime(), recvTime, ((CuckooMsg) msg).getHops(), ((CuckooMsg) msg).getType(), mk, io, fid, tid, redt, ((RplStatusErrMsg) msg).getAcTime(), ((RplStatusErrMsg) msg).getAcHops());
			} else {
				trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(msg), 0, ((CuckooMsg) msg).getSTime(), recvTime, ((CuckooMsg) msg).getHops(), ((CuckooMsg) msg).getType(), mk, io, fid, tid, redt, 0, 0);
			}
		}/*else if(msg instanceof ServerRequest) {
			trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(msg), 0, 0, 0, 0, ((BogusServerMsg) msg).getType(), mk, io, fid, tid, redt, 0, 0);
		}*/
		trafficXMLDB.addTrafficLog(trafficLog);
	}
	
	public void record(ServerMessage servermsg) {
		TrafficLog trafficLog = null;
		boolean io = servermsg.getIO();
		if(io == true) {
			trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(servermsg), 0, servermsg.getSendTime(), servermsg.getRecvTime(), 0, servermsg.getType(), "s", "i", "server", servermsg.getUserId(), "n", 0, 0);
		} else {
			trafficLog = new TrafficLog(Calendar.getInstance().getTimeInMillis(), sizeCalculator.calcSize(servermsg), 0, servermsg.getSendTime(), servermsg.getRecvTime(), 0, servermsg.getType(), "s", "o", servermsg.getUserId(), "server", "n", 0, 0);
		}
		trafficXMLDB.addTrafficLog(trafficLog);
	}
}
