package twclientapi;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.Calendar;

import servermsg.RequestRecentUpdates;
import servermsg.RequestUploadStatus;
import servermsg.ResponseAcknowledgement;
import servermsg.ResponseRecentUpdates;
import servermsg.ServerMessage;

import toolbox.TrafficAnalysisRecorder;

import application.Profile;
import application.Status;


public class TwClientAPI {
	private static final double NUM_SERVERS = 4.0;
	private static final String SERVER_SELECTION = "./server_config";
	
	/*private static String IP = "localhost";
	private static int PORT = 9000;*/
	
	private Profile userProfile;
	private TrafficAnalysisRecorder trafficRecorder = null;
	
	public TwClientAPI(Profile up) {
		userProfile = up;
	}
	
	public TwClientAPI(Profile up, TrafficAnalysisRecorder recorder) {
		this(up);
		trafficRecorder = recorder;
	}

	/*
	public Status[] requestRecentUpdates() {
		RecentUpdatesRequest  request  = new RecentUpdatesRequest(userProfile.getUserId(), Calendar.getInstance().getTimeInMillis(), 1, null, 0);
		ResponseRecentUpdates response = (ResponseRecentUpdates) sendRequest(request);
		return getStatusesFromRequestMsg(response);
	}
	*/
	
	public Status[] requestRecentUpdates(long updatetime) {
		long t = Calendar.getInstance().getTimeInMillis();
		RequestRecentUpdates request  = new RequestRecentUpdates(userProfile.getUserId(), t, t, 1, null, updatetime);
		ResponseRecentUpdates response = (ResponseRecentUpdates) sendRequest(request);
		return getStatusesFromRequestMsg(response);
	}
	
	/*
	public Status[] requestUpdates(String objId) {
		RecentUpdatesRequest  request  = new RecentUpdatesRequest(userProfile.getUserId(), Calendar.getInstance().getTimeInMillis(), 2, objId, 0);
		ResponseRecentUpdates response = (ResponseRecentUpdates) sendRequest(request);
		return getStatusesFromRequestMsg(response);
	}
	
	public Status[] requestUpdates(String objId, long updatetime) {
		RecentUpdatesRequest request  = new RecentUpdatesRequest(userProfile.getUserId(), Calendar.getInstance().getTimeInMillis(), 3, objId, updatetime);
		ResponseRecentUpdates response = (ResponseRecentUpdates) sendRequest(request);
		return getStatusesFromRequestMsg(response);
	}
	*/
	//used for requestRecentUpdates
	private Status[] getStatusesFromRequestMsg(ResponseRecentUpdates response) {
		if(response == null) {
			return null;
		} else {
			return response.getStatusArray();
		}
	}
	
	public String uploadStatus(String text) {
		long t = Calendar.getInstance().getTimeInMillis();
		RequestUploadStatus request = new RequestUploadStatus(userProfile.getUserId(), t, t, text);
		ResponseAcknowledgement response = (ResponseAcknowledgement) sendRequest(request);
		return response.getText();
	}
	
	private ServerMessage sendRequest(ServerMessage request) {
		ObjectOutputStream oos = null;
		ObjectInputStream ois = null;
		ServerMessage response = null;
		
		try {
			///////////////get the IP and PORT/////////////////
			BufferedReader servInput = new BufferedReader(new FileReader(SERVER_SELECTION));
			int lineNum = (int) (Math.random() * NUM_SERVERS);
			String servLine = null;
			while((servLine = servInput.readLine()) != null) {
				if(lineNum == 0) {
					servLine = servLine.trim();
					break;
				} else {
					lineNum --;
				}
			}
			String[] split = servLine.split("@");
			
			//System.out.println(split[0]);
			//System.out.println(split[1]);
			int port = Integer.valueOf(split[0]);
			if(request instanceof RequestUploadStatus) {
				port += 1;
			}
			
			// open a socket connection
			Socket socket = new Socket(split[1], port);
		
			// open I/O streams for objects
			oos = new ObjectOutputStream(socket.getOutputStream());
			ois = new ObjectInputStream(socket.getInputStream());
			
			// write the objects to the server
			oos.writeObject(request);
			oos.flush();
			if(trafficRecorder != null) {
				trafficRecorder.record(request);
			}
			
			// read an object from the server
			response = (ServerMessage) ois.readObject();
			if(trafficRecorder != null) {
				trafficRecorder.record(response);
			}

			oos.close();
			ois.close();
		} catch(Exception e) {
			System.out.println(e.getMessage());
		}
		return response;
	}
}
